<?php

abstract class DispletRetsIdxSettings extends DispletRetsIdxPlugin{
	protected static $settings_option = 'displet_rets_idx_options';
	protected static $version_option = 'displet_rets_idx_version';
	protected static $page_option = 'displet_rets_idx_page_options';
	protected static $field_option = 'displet_rets_idx_field_options';
	protected static $authenticate_option = 'displet_rets_idx_authenticate_options';
	protected static $cron_option = 'displet_rets_idx_cron_options';

	public function init(){
		if (is_admin()) {	
			require_once('settings/settings-config.php');
		}
	}

	public function get_defaults(){
	    $default_page = get_page_by_path('rets');
		$admin_email = get_bloginfo('admin_email');
		$displetretsidx_option = array(
			'address_format' => '%%street_number%% %%street_pre_direction%% %%street_name%% %%street_post_direction%% #%%unit%%',
			'allowed_public_views' => '0',
			'allow_sitewide_registration' => 0,
			'color_scheme' => 'default',
			'duty_agent_use_listing_agent_email' => 0,
			'email' => $admin_email,
			'email_activity' => 0,
			'email_from_address' => $admin_email,
			'email_from_name' => get_bloginfo('name'),
			'email_signature' => DispletRetsIdxEmailTemplates::get_signature(),
			'email_title_new_registration_to_user' => DispletRetsIdxEmailTemplates::get_new_registration_title_to_user(),
			'email_template_new_registration_to_user' => DispletRetsIdxEmailTemplates::get_new_registration_message_to_user(),
			'email_title_new_registration_to_admin' => DispletRetsIdxEmailTemplates::get_new_registration_title_to_admin(),
			'email_template_new_registration_to_admin' => DispletRetsIdxEmailTemplates::get_new_registration_message_to_admin(),
			'email_title_property_showing_to_user' => DispletRetsIdxEmailTemplates::get_property_showing_title_to_user(),
			'email_template_property_showing_to_user' => DispletRetsIdxEmailTemplates::get_property_showing_message_to_user(),
			'email_title_property_showing_to_admin' => DispletRetsIdxEmailTemplates::get_property_showing_title_to_admin(),
			'email_template_property_showing_to_admin' => DispletRetsIdxEmailTemplates::get_property_showing_message_to_admin(),
			'email_title_email_friend_to_user' => DispletRetsIdxEmailTemplates::get_email_friend_title_to_user(),
			'email_template_email_friend_to_user' => DispletRetsIdxEmailTemplates::get_email_friend_message_to_user(),
			'email_title_email_friend_to_admin' => DispletRetsIdxEmailTemplates::get_email_friend_title_to_admin(),
			'email_template_email_friend_to_admin' => DispletRetsIdxEmailTemplates::get_email_friend_message_to_admin(),
			'email_title_assigned_lead' => DispletRetsIdxEmailTemplates::get_assigned_lead_title(),
			'email_template_assigned_lead' => DispletRetsIdxEmailTemplates::get_assigned_lead_message(),
			'email_title_user_activity' => DispletRetsIdxEmailTemplates::get_activity_report_title(),
			'email_template_user_activity' => DispletRetsIdxEmailTemplates::get_activity_report_message(),
			'exclude_referred_visitors' => 1,
			'include_disclaimer_image' => 0,
			'include_feed_image_trans_date' => 1,
			'include_foreclosure' => 1,
			'include_latitude' => 1,
			'include_list_date' => 1,
			'include_listing_agent' => 0,
			'include_listing_agent_id' => 1,
			'include_listing_agent_pdp' => 0,
			'include_listing_office' => 1,
			'include_listing_office_id' => 1,
			'include_listing_office_pdp' => 1,
			'include_longitude' => 1,
			'include_mls_number' => 1,
			'include_modified' => 1,
			'include_price_navigation' => 1,
			'include_price_per_square_feet' => 1,
			'include_property_type_navigation' => 0,
			'include_property_type_sorting' => 1,
			'include_short_sale' => 1,
			'include_stats' => 1,
			'include_subdivision' => 1,
			'include_subdivision_pdp' => 1,
			'include_sysid' => 1,
			'include_working_with_realtor' => 0,
			'listings_orientation' => 'gallery',
			'listings_sort' => 'price-descending',
			'listings_per_page' => '10',
			'mobile_version' => 0,
			'oodle_category' => 'housing/sale',
			'oodle_region' => 'usa',
			'map_variance' => '2',
			'no_link' => 0,
			'phone' => '',
			'property_details_page_id' => $default_page->ID,
			'quick_start_parent_menu' => 'new',
			'quick_start_parent_page' => 'new',
			'quick_start_type' => 'zip',
			'require_phone' => 0,
			'require_working_with_realtor' => 0,
			'search_results_page_id' => $default_page->ID,
			'search_values_property_types' => array(
				'House' => 'true',
				'Condo' => 'true',
				'Lease' => 'true',
				'Land' => 'true',
				'Multi' => 'true',
				'Ranch' => 'true'
			),
			'search_values_min_price' => '100',
			'search_values_max_price' => '1000',
			'search_values_price_increment' => '50',
			'search_values_beds_min' => '1',
			'search_values_beds_max' => '6',
			'search_values_baths_min' => '1',
			'search_values_baths_max' => '6',
			'search_values_min_square_feet' => '500',
			'search_values_max_square_feet' => '5000',
			'search_values_square_feet_increment' => '500',
			'search_values_min_acres' => '.25',
			'search_values_max_acres' => '5',
			'search_values_acres_increment' => '.25',
			'showing_request_require_phone' => 1
		);
		return $displetretsidx_option;
	}

    public function register_default_settings(){
	    $has_default_page = get_page_by_path('rets');
	    if (!$has_default_page) {    
	        $displet_default_rets_idx_page = array(
	          'post_name' => 'rets',
	          'post_title' => 'Real Estate',
	          'post_type' => 'page',
	          'post_status' => 'publish'
	        );
	        wp_insert_post($displet_default_rets_idx_page);
	    }
	    $displetretsidx_option = self::get_defaults();
		add_option('displet_rets_idx_options', $displetretsidx_option);
	}

	public function get_option($name = 'settings'){
		$option_name = self::get_option_name($name);
		if (!empty($option_name)) {
			return get_option($option_name);
		}
	}

	public function update_option($name, $value){
		$option_name = self::get_option_name($name);
		if (!empty($option_name)) {
			update_option($option_name, $value);
		}
	}

	private function get_option_name($name){
		if ($name == 'settings') {
			return self::$settings_option;
		}
		else if ($name == 'version') {
			return self::$version_option;
		}
		else if ($name == 'page') {
			return self::$page_option;
		}
		else if ($name == 'fields') {
			return self::$field_option;
		}
		else if ($name == 'authenticate') {
			return self::$authenticate_option;
		}
		else if ($name == 'cron') {
			return self::$cron_option;
		}
		return false;
	}

	public function set_cookie(){
		check_ajax_referer('displet_set_cookie_nonce');
		if (isset($_POST['action']) && $_POST['action'] == 'displet_set_cookie_request'){
			if (!empty($_POST['name']) && !empty($_POST['value'])) {
        		setcookie($_POST['name'], $_POST['value'], 0, '/');
			}
		}
		die();
	}

	public function get_cookies(){
		check_ajax_referer('displet_get_cookies_nonce');
		if (isset($_POST['action']) && $_POST['action'] == 'displet_get_cookies_request'){
			$cookies = new stdClass();
			$cookies->last_viewed_sort = $_COOKIE['displetretsidx_last_viewed_sort'];
			$cookies->last_viewed_hash = $_COOKIE['displetretsidx_last_viewed_hash'];
			$cookies->last_viewed_orientation = $_COOKIE['displetretsidx_last_viewed_orientation'];
			$cookies->last_viewed_count = $_COOKIE['displetretsidx_last_viewed_count'];
			$cookies->referring_site = $_COOKIE['displetretsidx_referring_site'];
			echo json_encode($cookies);
		}
		die();
	}

	public function set_field_options(){
		// Making a change here requires updating version
		$displetretsidx_option = DispletRetsIdxSettings::get_option();
		$displetretsidx_field_option = DispletRetsIdxSettings::get_option('fields');
		$is_new_api_key = DispletRetsIdxUtilities::is_new_api_key($displetretsidx_field_option, $displetretsidx_option);
		if ($is_new_api_key || empty($displetretsidx_field_option['version']) || $displetretsidx_field_option['version'] < 1) {
			$displetretsidx_field_option['displet_app_key'] = $displetretsidx_option['displet_app_key'];
			$displetretsidx_field_option['property_type'] = DispletRetsIdxQuery::get_field_values('property_type', $displetretsidx_option['displet_app_key']);
			$displetretsidx_field_option['school_district'] = DispletRetsIdxQuery::get_field_values('school_district', $displetretsidx_option['displet_app_key']);
			$displetretsidx_field_option['area_mls_defined'] = DispletRetsIdxQuery::get_field_values('area_mls_defined', $displetretsidx_option['displet_app_key']);
			$displetretsidx_field_option['city'] = DispletRetsIdxQuery::get_field_values('city', $displetretsidx_option['displet_app_key']);
			$displetretsidx_field_option['status'] = DispletRetsIdxQuery::get_field_values('status', $displetretsidx_option['displet_app_key']);
			if (!empty($displetretsidx_field_option['property_type'])) {
				foreach ($displetretsidx_field_option['property_type'] as $property_type) {
					$displetretsidx_option['search_values_property_types'][$property_type] = 'true';
				}
			}
			$displetretsidx_field_option['version'] = 1;
			update_option('displet_rets_idx_options', $displetretsidx_option);
			update_option('displet_rets_idx_field_options', $displetretsidx_field_option);
		}
	}

	public function check_page_options(){
		$displetretsidx_option = DispletRetsIdxSettings::get_option();
		$page_option = DispletRetsIdxSettings::get_option('page');
		$is_new_search_results_page = DispletRetsIdxUtilities::is_option_new('search_results_page_id', $page_option, $displetretsidx_option);
		$is_new_property_details_page = DispletRetsIdxUtilities::is_option_new('property_details_page_id', $page_option, $displetretsidx_option);
		if ($is_new_search_results_page || $is_new_property_details_page) {
			DispletRetsIdxUrlStructure::reset_flush_rewrite_check();
			$page_option = array(
				'search_results_page_id' => $displetretsidx_option['search_results_page_id'],
				'property_details_page_id' => $displetretsidx_option['property_details_page_id']
			);
			self::update_option('page', $page_option);
		}
	}

    public function update_version_ajax(){
        check_ajax_referer('displet_update_version_nonce');
        if (isset($_POST['action']) && $_POST['action'] == 'displet_update_version_request'){
    	   DispletRetsIdxSettings::update_option('version', parent::$plugin_version);
        }
        die();
    }

	public function update_api_website_values($api_key, $modify_token, $email_address){
		$home_url = home_url();
		$trimmed_domain = DispletRetsIdxUtilities::remove_url_suffixes(DispletRetsIdxUtilities::remove_url_prefixes($home_url));
		$query_url = parent::$api_url . '/api_apps/' . $api_key;
		$query_args = array(
			'method' => 'PUT',
			'timeout' => 10,
			'headers' => array(
				'referer' => $home_url
			),
			'body' => array(
				'email' => $email_address,
				'domain' => $trimmed_domain,
				'authentication_token' => $api_key,
				'modify_token' => $modify_token
			)
		);
		$json_array = wp_remote_post($query_url, $query_args);
		if (is_array($json_array) && !empty($json_array['body'])) {
			$json_response = json_decode($json_array['body']);
			if (!empty($json_response->success)) {
				return $home_url;
			}
		}
		return false;
	}

	public function authenticate_api_key(){
		// Making a change here requires updating version
		$displetretsidx_option = self::get_option();
		$displetretsidx_authenticate_option = self::get_option('authenticate');
		$is_new_api_key = DispletRetsIdxUtilities::is_new_api_key($displetretsidx_authenticate_option, $displetretsidx_option);
		$is_new_email_address = DispletRetsIdxUtilities::is_new_email_address($displetretsidx_authenticate_option, $displetretsidx_option);
		$is_new_api_token = DispletRetsIdxUtilities::is_new_api_token($displetretsidx_authenticate_option, $displetretsidx_option);
		if ($is_new_api_key || $is_new_email_address || $is_new_api_token || (empty($displetretsidx_option['email_from_address']) && !empty($displetretsidx_authenticate_option['email_from_address'])) || empty($displetretsidx_authenticate_option['version']) || $displetretsidx_authenticate_option['version'] < 1) {
			$displetretsidx_authenticate_option['version'] = 1;
			$displetretsidx_authenticate_option['displet_app_key'] = $displetretsidx_option['displet_app_key'];
			$displetretsidx_authenticate_option['displet_app_token'] = $displetretsidx_option['displet_app_token'];
			$displetretsidx_authenticate_option['email_from_address'] = $displetretsidx_option['email_from_address'];
			$email_address = (!empty($displetretsidx_option['email_from_address'])) ? $displetretsidx_option['email_from_address'] : get_bloginfo('admin_email');
			$displetretsidx_authenticate_option['authenticated_url'] = self::update_api_website_values($displetretsidx_option['displet_app_key'], $displetretsidx_option['displet_app_token'], $email_address);
			update_option(self::$authenticate_option, $displetretsidx_authenticate_option);
			if (empty($displetretsidx_authenticate_option['authenticated_url'])) {
				$displetretsidx_option['displet_app_token'] = '';
				update_option(self::$settings_option, $displetretsidx_option);
			}
		}
	}
}

add_action('init', array('DispletRetsIdxSettings', 'init'));
add_action('wp_ajax_displet_set_cookie_request', array('DispletRetsIdxSettings', 'set_cookie'));
add_action('wp_ajax_nopriv_displet_set_cookie_request', array('DispletRetsIdxSettings', 'set_cookie'));
add_action('wp_ajax_displet_get_cookies_request', array('DispletRetsIdxSettings', 'get_cookies'));
add_action('wp_ajax_nopriv_displet_get_cookies_request', array('DispletRetsIdxSettings', 'get_cookies'));
add_action('wp_ajax_displet_update_version_request', array('DispletRetsIdxSettings', 'update_version_ajax'));
add_action('wp_ajax_nopriv_displet_update_version_request', array('DispletRetsIdxSettings', 'update_version_ajax'));

?>